@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -465,7 +465,7 @@ gpm_brightness_hal_init (GpmBrightnessHal *brightness)
 
 	/* We only want first laptop_panel object (should only be one) */
 	brightness->priv->udi = g_strdup (names[0]);
-	hal_manager_free_capability (names);
+	g_strfreev (names);
 
 	device = hal_device_new ();
 	hal_device_set_udi (device, brightness->priv->udi);
