@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -42,7 +42,6 @@ static void     gpm_engine_finalize   (GObject	  *object);
 #define GPM_ENGINE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_ENGINE, GpmEnginePrivate))
 #define GPM_ENGINE_RESUME_DELAY		2*1000
 #define GPM_ENGINE_WARN_ACCURACY	20
-#define GPM_USE_COMPOSITE		0
 
 struct GpmEnginePrivate
 {
@@ -84,6 +83,7 @@ static gpointer gpm_engine_object = NULL;
 
 G_DEFINE_TYPE (GpmEngine, gpm_engine, G_TYPE_OBJECT)
 
+static DkpDevice *gpm_engine_get_composite_device (GpmEngine *engine, DkpDevice *original_device);
 static DkpDevice *gpm_engine_update_composite_device (GpmEngine *engine, DkpDevice *original_device);
 
 typedef enum {
@@ -130,9 +130,9 @@ gpm_engine_get_warning_percentage (GpmEngine *engine, DkpDevice *device)
 	}
 	if (percentage <= engine->priv->action_percentage)
 		return GPM_ENGINE_WARNING_ACTION;
-	else if (percentage <= engine->priv->critical_percentage)
+	if (percentage <= engine->priv->critical_percentage)
 		return GPM_ENGINE_WARNING_CRITICAL;
-	else if (percentage <= engine->priv->low_percentage)
+	if (percentage <= engine->priv->low_percentage)
 		return GPM_ENGINE_WARNING_LOW;
 	return GPM_ENGINE_WARNING_NONE;
 }
@@ -160,9 +160,9 @@ gpm_engine_get_warning_time (GpmEngine *engine, DkpDevice *device)
 
 	if (time_to_empty <= engine->priv->action_time)
 		return GPM_ENGINE_WARNING_ACTION;
-	else if (time_to_empty <= engine->priv->critical_time)
+	if (time_to_empty <= engine->priv->critical_time)
 		return GPM_ENGINE_WARNING_CRITICAL;
-	else if (time_to_empty <= engine->priv->low_time)
+	if (time_to_empty <= engine->priv->low_time)
 		return GPM_ENGINE_WARNING_LOW;
 	return GPM_ENGINE_WARNING_NONE;
 }
@@ -290,7 +290,7 @@ gpm_engine_get_icon_priv (GpmEngine *engine, DkpDeviceType device_type, GpmEngin
 
 		/* if battery then use composite device to cope with multiple batteries */
 		if (type == DKP_DEVICE_TYPE_BATTERY)
-			device = gpm_engine_update_composite_device (engine, device);
+			device = gpm_engine_get_composite_device (engine, device);
 
 		warning_temp = GPOINTER_TO_INT(g_object_get_data (G_OBJECT(device), "engine-warning-old"));
 		if (type == device_type && is_present) {
@@ -348,6 +348,26 @@ gpm_engine_get_icon (GpmEngine *engine)
 		return NULL;
 	}
 
+	/* we try CRITICAL: BATTERY, UPS, MOUSE, KEYBOARD */
+	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_BATTERY, GPM_ENGINE_WARNING_LOW, FALSE);
+	if (icon != NULL)
+		return icon;
+	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_UPS, GPM_ENGINE_WARNING_LOW, FALSE);
+	if (icon != NULL)
+		return icon;
+	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_MOUSE, GPM_ENGINE_WARNING_LOW, FALSE);
+	if (icon != NULL)
+		return icon;
+	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_KEYBOARD, GPM_ENGINE_WARNING_LOW, FALSE);
+	if (icon != NULL)
+		return icon;
+
+	/* policy */
+	if (engine->priv->icon_policy == GPM_ICON_POLICY_LOW) {
+		egg_debug ("no devices low, so no icon will be displayed.");
+		return NULL;
+	}
+
 	/* we try (DIS)CHARGING: BATTERY, UPS */
 	icon = gpm_engine_get_icon_priv (engine, DKP_DEVICE_TYPE_BATTERY, GPM_ENGINE_WARNING_NONE, TRUE);
 	if (icon != NULL)
@@ -496,7 +516,7 @@ gpm_engine_conf_key_changed_cb (GConfClient *conf, guint cnxn_id, GConfEntry *en
 
 		/* do we want to display the icon in the tray */
 		icon_policy = gconf_client_get_string (conf, GPM_CONF_UI_ICON_POLICY, NULL);
-		engine->priv->icon_policy = gpm_tray_icon_mode_from_string (icon_policy);
+		engine->priv->icon_policy = gpm_icon_policy_from_string (icon_policy);
 		g_free (icon_policy);
 
 		/* perhaps change icon */
@@ -541,6 +561,43 @@ gpm_engine_device_check_capacity (GpmEngine *engine, DkpDevice *device)
 	return TRUE;
 }
 
+/**
+ * gpm_engine_get_composite_device:
+ **/
+static DkpDevice *
+gpm_engine_get_composite_device (GpmEngine *engine, DkpDevice *original_device)
+{
+	guint battery_devices = 0;
+	GPtrArray *array;
+	DkpDevice *device;
+	DkpDeviceType type;
+	guint i;
+
+	/* find out how many batteries in the system */
+	array = engine->priv->array;
+	for (i=0;i<array->len;i++) {
+		device = g_ptr_array_index (engine->priv->array, i);
+		g_object_get (device,
+			      "type", &type,
+			      NULL);
+		if (type == DKP_DEVICE_TYPE_BATTERY)
+			battery_devices++;
+	}
+
+	/* just use the original device if only one primary battery */
+	if (battery_devices <= 1) {
+		egg_debug ("using original device as only one primary battery");
+		device = original_device;
+		goto out;
+	}
+
+	/* use the composite device */
+	device = engine->priv->battery_composite;
+out:
+	/* return composite device or original device */
+	return device;
+}
+
 /**
  * gpm_engine_update_composite_device:
  **/
@@ -560,7 +617,7 @@ gpm_engine_update_composite_device (GpmEngine *engine, DkpDevice *original_devic
 	guint battery_devices = 0;
 	gboolean is_charging = FALSE;
 	gboolean is_discharging = FALSE;
-	gboolean is_fully_charged = FALSE;
+	gboolean is_fully_charged = TRUE;
 	GPtrArray *array;
 	DkpDevice *device;
 	DkpDeviceState state;
@@ -594,8 +651,8 @@ gpm_engine_update_composite_device (GpmEngine *engine, DkpDevice *original_devic
 			is_charging = TRUE;
 		if (state == DKP_DEVICE_STATE_DISCHARGING)
 			is_discharging = TRUE;
-		if (state == DKP_DEVICE_STATE_FULLY_CHARGED)
-			is_fully_charged = TRUE;
+		if (state != DKP_DEVICE_STATE_FULLY_CHARGED)
+			is_fully_charged = FALSE;
 
 		/* sum up composite */
 		energy_total += energy;
@@ -645,7 +702,11 @@ gpm_engine_update_composite_device (GpmEngine *engine, DkpDevice *original_devic
 		      "percentage", percentage,
 		      "state", state,
 		      NULL);
-	dkp_device_print (device);
+	if (debug)
+		dkp_device_print (device);
+
+	/* force update of icon */
+	gpm_engine_recalculate_state_icon (engine);
 out:
 	/* return composite device or original device */
 	return device;
@@ -680,6 +741,7 @@ gpm_engine_device_add (GpmEngine *engine, DkpDevice *device)
 	g_object_set_data (G_OBJECT(device), "engine-state-old", GUINT_TO_POINTER(state));
 
 	if (type == DKP_DEVICE_TYPE_BATTERY) {
+		egg_debug ("updating because we added a device");
 		composite = gpm_engine_update_composite_device (engine, device);
 
 		/* get the same values for the composite device */
@@ -811,7 +873,6 @@ gpm_engine_device_removed_cb (DkpClient *client, DkpDevice *device, GpmEngine *e
 	ret = g_ptr_array_remove (engine->priv->array, device);
 	if (!ret)
 		return;
-	g_object_unref (device);
 	gpm_engine_recalculate_state (engine);
 }
 
@@ -831,6 +892,16 @@ gpm_engine_device_changed_cb (DkpClient *client, DkpDevice *device, GpmEngine *e
 	/* get device properties */
 	g_object_get (device,
 		      "type", &type,
+		      NULL);
+
+	/* if battery then use composite device to cope with multiple batteries */
+	if (type == DKP_DEVICE_TYPE_BATTERY) {
+		egg_debug ("updating because %s changed", dkp_device_get_object_path (device));
+		device = gpm_engine_update_composite_device (engine, device);
+	}
+
+	/* get device properties (may be composite) */
+	g_object_get (device,
 		      "state", &state,
 		      NULL);
 
@@ -851,10 +922,6 @@ gpm_engine_device_changed_cb (DkpClient *client, DkpDevice *device, GpmEngine *e
 		g_object_set_data (G_OBJECT(device), "engine-state-old", GUINT_TO_POINTER(state));
 	}
 
-	/* if battery then use composite device to cope with multiple batteries */
-	if (type == DKP_DEVICE_TYPE_BATTERY)
-		device = gpm_engine_update_composite_device (engine, device);
-
 	/* check the warning state has not changed */
 	warning_old = GPOINTER_TO_INT(g_object_get_data (G_OBJECT(device), "engine-warning-old"));
 	warning = gpm_engine_get_warning (engine, device);
@@ -878,20 +945,13 @@ gpm_engine_device_changed_cb (DkpClient *client, DkpDevice *device, GpmEngine *e
 
 /**
  * gpm_engine_get_devices:
+ *
+ * Return value: the DkpDevice array, free with g_ptr_array_unref()
  **/
-const GPtrArray *
+GPtrArray *
 gpm_engine_get_devices (GpmEngine *engine)
 {
-	guint i;
-	DkpDevice *device;
-	GPtrArray *array = engine->priv->array;
-
-	for (i=0; i<array->len; i++) {
-		device = g_ptr_array_index (array, i);
-		g_object_ref (G_OBJECT(device));
-	}
-
-	return engine->priv->array;
+	return g_ptr_array_ref (engine->priv->array);
 }
 
 /**
@@ -941,7 +1001,6 @@ phone_device_removed_cb (GpmPhone *phone, guint idx, GpmEngine *engine)
 
 		if (type == DKP_DEVICE_TYPE_PHONE) {
 			g_ptr_array_remove_index (engine->priv->array, i);
-			g_object_unref (device);
 			break;
 		}
 	}
@@ -999,7 +1058,7 @@ gpm_engine_init (GpmEngine *engine)
 
 	engine->priv = GPM_ENGINE_GET_PRIVATE (engine);
 
-	engine->priv->array = g_ptr_array_new ();
+	engine->priv->array = g_ptr_array_new_with_free_func (g_object_unref);
 	engine->priv->client = dkp_client_new ();
 	g_signal_connect (engine->priv->client, "device-added",
 			  G_CALLBACK (gpm_engine_device_added_cb), engine);
@@ -1036,7 +1095,7 @@ gpm_engine_init (GpmEngine *engine)
 
 	/* do we want to display the icon in the tray */
 	icon_policy = gconf_client_get_string (engine->priv->conf, GPM_CONF_UI_ICON_POLICY, NULL);
-	engine->priv->icon_policy = gpm_tray_icon_mode_from_string (icon_policy);
+	engine->priv->icon_policy = gpm_icon_policy_from_string (icon_policy);
 	g_free (icon_policy);
 
 	/* get percentage policy */
@@ -1104,7 +1163,7 @@ gpm_engine_class_init (GpmEngineClass *klass)
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GpmEngineClass, fully_charged),
-			      NULL, NULL, g_cclosure_marshal_VOID__UINT,
+			      NULL, NULL, g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1, G_TYPE_POINTER);
 	signals [DISCHARGING] =
 		g_signal_new ("discharging",
@@ -1151,9 +1210,7 @@ gpm_engine_finalize (GObject *object)
 	engine = GPM_ENGINE (object);
 	engine->priv = GPM_ENGINE_GET_PRIVATE (engine);
 
-	g_ptr_array_foreach (engine->priv->array, (GFunc) g_object_unref, NULL);
-	g_ptr_array_free (engine->priv->array, TRUE);
-
+	g_ptr_array_unref (engine->priv->array);
 	g_object_unref (engine->priv->client);
 	g_object_unref (engine->priv->phone);
 	g_object_unref (engine->priv->battery_composite);
