@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -91,6 +91,7 @@ struct GpmManagerPrivate
 	guint32			 screensaver_lid_throttle_id;
 	DkpClient		*client;
 	gboolean		 on_battery;
+	gboolean		 just_resumed;
 	GtkStatusIcon		*status_icon;
 	gboolean		 supports_notification_actions;
 	NotifyNotification	*notification;
@@ -415,39 +416,120 @@ out:
 	return ret;
 }
 
+
+/**
+ * gpm_manager_sleep_failure_response_cb:
+ **/
+static void
+gpm_manager_sleep_failure_response_cb (GtkDialog *dialog, gint response_id, GpmManager *manager)
+{
+	GdkScreen *screen;
+	GtkWidget *dialog_error;
+	GError *error = NULL;
+	gboolean ret;
+	gchar *uri = NULL;
+
+	/* user clicked the help button */
+	if (response_id == GTK_RESPONSE_HELP) {
+		uri = gconf_client_get_string (manager->priv->conf, GPM_CONF_NOTIFY_SLEEP_FAILED_URI, NULL);
+		screen = gdk_screen_get_default();
+		ret = gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
+		if (!ret) {
+			dialog_error = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+							       "Failed to show uri %s", error->message);
+			gtk_dialog_run (GTK_DIALOG (dialog_error));
+			g_error_free (error);
+		}
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+	g_free (uri);
+}
+
+/**
+ * gpm_manager_sleep_failure:
+ **/
+static void
+gpm_manager_sleep_failure (GpmManager *manager, gboolean is_suspend, const gchar *detail)
+{
+	gboolean show_sleep_failed;
+	GString *string = NULL;
+	const gchar *title;
+	gchar *uri = NULL;
+	const gchar *icon;
+	GtkWidget *dialog;
+
+	/* only show this if specified in gconf */
+	show_sleep_failed = gconf_client_get_bool (manager->priv->conf, GPM_CONF_NOTIFY_SLEEP_FAILED, NULL);
+
+	egg_debug ("sleep failed");
+	gpm_manager_play (manager, GPM_MANAGER_SOUND_SUSPEND_ERROR, TRUE);
+
+	/* only emit if in GConf */
+	if (!show_sleep_failed)
+		goto out;
+
+	/* TRANSLATORS: window title: there was a problem putting the machine to sleep */
+	string = g_string_new ("");
+	if (is_suspend) {
+		/* TRANSLATORS: message text */
+		g_string_append (string, _("Your computer failed to suspend."));
+		/* TRANSLATORS: title text */
+		title = _("Failed to suspend");
+		icon = GPM_STOCK_SUSPEND;
+	} else {
+		/* TRANSLATORS: message text */
+		g_string_append (string, _("Your computer failed to hibernate."));
+		/* TRANSLATORS: title text */
+		title = _("Failed to hibernate");
+		icon = GPM_STOCK_HIBERNATE;
+	}
+
+	/* TRANSLATORS: message text */
+	g_string_append_printf (string, "\n\n%s %s", _("The failure was reported as:"), detail);
+
+	/* show modal dialog */
+	dialog = gtk_message_dialog_new_with_markup (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+						     GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
+						     "<span size='larger'><b>%s</b></span>", title);
+	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog), "%s", string->str);
+	gtk_window_set_icon_name (GTK_WINDOW(dialog), icon);
+
+	/* show a button? */
+	uri = gconf_client_get_string (manager->priv->conf, GPM_CONF_NOTIFY_SLEEP_FAILED_URI, NULL);
+	if (uri != NULL && uri[0] != '\0') {
+		/* TRANSLATORS: button text, visit the suspend help website */
+		gtk_dialog_add_button (GTK_DIALOG (dialog), _("Visit help page"), GTK_RESPONSE_HELP);
+	}
+
+	/* wait async for close */
+	gtk_widget_show (dialog);
+	g_signal_connect (dialog, "response", G_CALLBACK (gpm_manager_sleep_failure_response_cb), manager);
+out:
+	g_free (uri);
+	g_string_free (string, TRUE);
+}
+
 /**
  * gpm_manager_action_suspend:
  **/
 static gboolean
 gpm_manager_action_suspend (GpmManager *manager, const gchar *reason)
 {
-	gboolean allowed;
+	gboolean ret;
 	GError *error = NULL;
 
-	/* check if the admin has disabled */
-	allowed = gconf_client_get_bool (manager->priv->conf, GPM_CONF_CAN_SUSPEND, NULL);
-	if (allowed == FALSE) {
-		/* error msg as disabled in gconf */
-		gpm_manager_notify (manager, &manager->priv->notification,
-				    /* TRANSLATORS: the action was not done */
-				    _("Action disallowed"),
-				    /* TRANSLATORS: admin has disabled ability to do this */
-				    _("Suspend support has been disabled. Contact your administrator for more details."),
-				    GPM_MANAGER_NOTIFY_TIMEOUT_SHORT,
-				    GPM_STOCK_APP_ICON,
-				    NOTIFY_URGENCY_NORMAL);
-		return FALSE;
-	}
-
 	/* check to see if we are inhibited */
 	if (gpm_manager_is_inhibit_valid (manager, FALSE, "suspend") == FALSE)
 		return FALSE;
 
 	egg_debug ("suspending, reason: %s", reason);
-	gpm_control_suspend (manager->priv->control, &error);
-	gpm_button_reset_time (manager->priv->button);
-	if (error != NULL)
+	ret = gpm_control_suspend (manager->priv->control, &error);
+	if (!ret) {
+		gpm_manager_sleep_failure (manager, TRUE, error->message);
 		g_error_free (error);
+	}
+	gpm_button_reset_time (manager->priv->button);
 	return TRUE;
 }
 
@@ -457,33 +539,20 @@ gpm_manager_action_suspend (GpmManager *manager, const gchar *reason)
 static gboolean
 gpm_manager_action_hibernate (GpmManager *manager, const gchar *reason)
 {
-	gboolean allowed;
+	gboolean ret;
 	GError *error = NULL;
 
-	/* check if the admin has disabled */
-	allowed = gconf_client_get_bool (manager->priv->conf, GPM_CONF_CAN_HIBERNATE, NULL);
-	if (allowed == FALSE) {
-		/* error msg as disabled in gconf */
-		gpm_manager_notify (manager, &manager->priv->notification,
-				    /* TRANSLATORS: the action was not done */
-				    _("Action disallowed"),
-				    /* TRANSLATORS: admin has disabled ability to do this */
-				    _("Hibernate support has been disabled. Contact your administrator for more details."),
-				    GPM_MANAGER_NOTIFY_TIMEOUT_SHORT,
-				    GPM_STOCK_APP_ICON,
-				    NOTIFY_URGENCY_NORMAL);
-		return FALSE;
-	}
-
 	/* check to see if we are inhibited */
 	if (gpm_manager_is_inhibit_valid (manager, FALSE, "hibernate") == FALSE)
 		return FALSE;
 
 	egg_debug ("hibernating, reason: %s", reason);
-	gpm_control_hibernate (manager->priv->control, &error);
-	gpm_button_reset_time (manager->priv->button);
-	if (error != NULL)
+	ret = gpm_control_hibernate (manager->priv->control, &error);
+	if (!ret) {
+		gpm_manager_sleep_failure (manager, TRUE, error->message);
 		g_error_free (error);
+	}
+	gpm_button_reset_time (manager->priv->button);
 	return TRUE;
 }
 
@@ -928,34 +997,6 @@ gpm_conf_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *e
 		gpm_manager_sync_policy_sleep (manager);
 }
 
-/**
- * gpm_manager_tray_icon_hibernate:
- * @manager: This class instance
- * @tray: The tray object
- *
- * The icon tray hibernate callback, which only should happen if both policy and
- * the inhibit states are valid.
- **/
-static void
-gpm_manager_tray_icon_hibernate (GpmManager *manager, GpmTrayIcon *tray)
-{
-	gpm_manager_action_hibernate (manager, "User clicked on tray");
-}
-
-/**
- * gpm_manager_tray_icon_suspend:
- * @manager: This class instance
- * @tray: The tray object
- *
- * The icon tray suspend callback, which only should happen if both policy and
- * the inhibit states are valid.
- **/
-static void
-gpm_manager_tray_icon_suspend (GpmManager *manager, GpmTrayIcon *tray)
-{
-	gpm_manager_action_suspend (manager, "User clicked on tray");
-}
-
 #if 0
 /**
  * gpm_manager_screensaver_auth_request_cb:
@@ -1266,93 +1307,6 @@ out:
 	return;
 }
 
-/**
- * gpm_manager_sleep_failure_response_cb:
- **/
-static void
-gpm_manager_sleep_failure_response_cb (GtkDialog *dialog, gint response_id, GpmManager *manager)
-{
-	GdkScreen *screen;
-	GtkWidget *dialog_error;
-	GError *error = NULL;
-	gboolean ret;
-	gchar *uri = NULL;
-
-	/* user clicked the help button */
-	if (response_id == GTK_RESPONSE_HELP) {
-		uri = gconf_client_get_string (manager->priv->conf, GPM_CONF_NOTIFY_SLEEP_FAILED_URI, NULL);
-		screen = gdk_screen_get_default();
-		ret = gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
-		if (!ret) {
-			dialog_error = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
-							       "Failed to show uri %s", error->message);
-			gtk_dialog_run (GTK_DIALOG (dialog_error));
-			g_error_free (error);
-		}
-	}
-
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-	g_free (uri);
-}
-
-/**
- * gpm_manager_sleep_failure_cb:
- **/
-static void
-gpm_manager_sleep_failure_cb (GpmControl *control, GpmControlAction action, GpmManager *manager)
-{
-	gboolean show_sleep_failed;
-	gchar *message = NULL;
-	gchar *title = NULL;
-	gchar *uri = NULL;
-	const gchar *icon;
-	GtkWidget *dialog;
-
-	/* only show this if specified in gconf */
-	show_sleep_failed = gconf_client_get_bool (manager->priv->conf, GPM_CONF_NOTIFY_SLEEP_FAILED, NULL);
-
-	egg_debug ("sleep failed");
-	gpm_manager_play (manager, GPM_MANAGER_SOUND_SUSPEND_ERROR, TRUE);
-
-	/* only emit if in GConf */
-	if (!show_sleep_failed)
-		goto out;
-
-	/* TRANSLATORS: window title: there was a problem putting the machine to sleep */
-	title = g_strdup_printf ("%s: %s", GPM_NAME, _("Sleep problem"));
-	if (action == GPM_CONTROL_ACTION_SUSPEND) {
-		/* TRANSLATORS: message text */
-		message = g_strdup_printf ("%s\n%s", _("Your computer failed to suspend."), _("Check the help file for common problems."));
-		icon = GPM_STOCK_SUSPEND;
-	} else {
-		/* TRANSLATORS: message text */
-		message = g_strdup_printf ("%s\n%s", _("Your computer failed to hibernate."), _("Check the help file for common problems."));
-		icon = GPM_STOCK_HIBERNATE;
-	}
-
-	/* show modal dialog */
-	dialog = gtk_message_dialog_new_with_markup (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-						     GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
-						     "<span size='larger'><b>%s</b></span>", title);
-	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog), "%s", message);
-	gtk_window_set_icon_name (GTK_WINDOW(dialog), icon);
-
-	/* show a button? */
-	uri = gconf_client_get_string (manager->priv->conf, GPM_CONF_NOTIFY_SLEEP_FAILED_URI, NULL);
-	if (uri != NULL && uri[0] != '\0') {
-		/* TRANSLATORS: button text, visit the suspend help website */
-		gtk_dialog_add_button (GTK_DIALOG (dialog), _("Visit help page"), GTK_RESPONSE_HELP);
-	}
-
-	/* wait async for close */
-	gtk_widget_show (dialog);
-	g_signal_connect (dialog, "response", G_CALLBACK (gpm_manager_sleep_failure_response_cb), manager);
-out:
-	g_free (uri);
-	g_free (title);
-	g_free (message);
-}
-
 /**
  * gpm_manager_engine_just_laptop_battery:
  */
@@ -1731,6 +1685,27 @@ gpm_manager_dpms_mode_changed_cb (GpmDpms *dpms, GpmDpmsMode mode, GpmManager *m
 	gpm_manager_update_dpms_throttle (manager);
 }
 
+/*
+ * gpm_manager_reset_just_resumed_cb
+ */
+static gboolean
+gpm_manager_reset_just_resumed_cb (gpointer user_data)
+{
+	GpmManager *manager = GPM_MANAGER (user_data);
+	manager->priv->just_resumed = FALSE;
+	return FALSE;
+}
+
+/**
+ * gpm_manager_control_resume_cb
+ **/
+static void
+gpm_manager_control_resume_cb (GpmControl *control, GpmControlAction action, GpmManager *manager)
+{
+	manager->priv->just_resumed = TRUE;
+	g_timeout_add_seconds (1, gpm_manager_reset_just_resumed_cb, manager);
+}
+
 /**
  * gpm_manager_console_kit_active_changed_cb:
  **/
@@ -1750,6 +1725,12 @@ gpm_manager_console_kit_active_changed_cb (EggConsoleKit *console, gboolean acti
 	if (!ret)
 		return;
 
+	/* lid state might not be accurate if just resumed. Don't do anything if
+	 * we've just resumed as we might end up re-suspending the machine due
+	 * to pm-utils and uswsusp changing the tty */
+	if (manager->priv->just_resumed)
+		return;
+
 	/* get ac state */
 	if (!manager->priv->on_battery) {
 		egg_debug ("Performing AC policy as become active when lid down");
@@ -1815,6 +1796,9 @@ gpm_manager_init (GpmManager *manager)
 	manager->priv->screensaver_dpms_throttle_id = 0;
 	manager->priv->screensaver_lid_throttle_id = 0;
 
+	/* init to not just_resumed */
+	manager->priv->just_resumed = FALSE;
+
 	/* don't apply policy when not active */
 	manager->priv->console = egg_console_kit_new ();
 	g_signal_connect (manager->priv->console, "active-changed",
@@ -1899,17 +1883,11 @@ gpm_manager_init (GpmManager *manager)
 	/* use the control object */
 	egg_debug ("creating new control instance");
 	manager->priv->control = gpm_control_new ();
-	g_signal_connect (manager->priv->control, "sleep-failure",
-			  G_CALLBACK (gpm_manager_sleep_failure_cb), manager);
+	g_signal_connect (manager->priv->control, "resume",
+			  G_CALLBACK (gpm_manager_control_resume_cb), manager);
 
 	egg_debug ("creating new tray icon");
 	manager->priv->tray_icon = gpm_tray_icon_new ();
-	g_signal_connect_object (G_OBJECT (manager->priv->tray_icon),
-				 "suspend", G_CALLBACK (gpm_manager_tray_icon_suspend),
-				 manager, G_CONNECT_SWAPPED);
-	g_signal_connect_object (G_OBJECT (manager->priv->tray_icon),
-				 "hibernate", G_CALLBACK (gpm_manager_tray_icon_hibernate),
-				 manager, G_CONNECT_SWAPPED);
 
 	/* keep a reference for the notifications */
 	manager->priv->status_icon = gpm_tray_icon_get_status_icon (manager->priv->tray_icon);
