@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef __GPMPHONE_H
@@ -49,24 +49,27 @@ typedef struct
 {
 	GObjectClass	parent_class;
 	void		(* device_added)		(GpmPhone	*phone,
-							 guint		 index);
+							 guint		 idx);
 	void		(* device_removed)		(GpmPhone	*phone,
-							 guint		 index);
+							 guint		 idx);
 	void		(* device_refresh)		(GpmPhone	*phone,
-							 guint		 index);
+							 guint		 idx);
 } GpmPhoneClass;
 
-GType		 gpm_phone_get_type		(void);
-GpmPhone	*gpm_phone_new			(void);
+GType		 gpm_phone_get_type			(void);
+GpmPhone	*gpm_phone_new				(void);
 
-gboolean	 gpm_phone_get_present		(GpmPhone	*phone,
-						 guint		 index);
-guint		 gpm_phone_get_percentage	(GpmPhone	*phone,
-						 guint		 index);
-gboolean	 gpm_phone_get_on_ac		(GpmPhone	*phone,
-						 guint		 index);
-guint		 gpm_phone_get_num_batteries	(GpmPhone	*phone);
-gboolean	 gpm_phone_coldplug		(GpmPhone	*phone);
+gboolean	 gpm_phone_get_present			(GpmPhone	*phone,
+							 guint		 idx);
+guint		 gpm_phone_get_percentage		(GpmPhone	*phone,
+							 guint		 idx);
+gboolean	 gpm_phone_get_on_ac			(GpmPhone	*phone,
+							 guint		 idx);
+guint		 gpm_phone_get_num_batteries		(GpmPhone	*phone);
+gboolean	 gpm_phone_coldplug			(GpmPhone	*phone);
+#ifdef EGG_TEST
+void		 gpm_phone_test				(gpointer	 data);
+#endif
 
 G_END_DECLS
 
