@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #include "config.h"
@@ -113,7 +113,7 @@ enum {
 static void
 gpm_stats_button_help_cb (GtkWidget *widget, gboolean data)
 {
-	//gpm_gnome_help ("update-log");
+	gpm_help_display ("statistics");
 }
 
 /**
@@ -308,22 +308,14 @@ gpm_stats_bool_to_text (gboolean ret)
 static gchar *
 gpm_stats_get_printable_device_path (DkpDevice *device)
 {
-	const gchar *prefix = "/org/freedesktop/DeviceKit/Power/devices/";
 	const gchar *object_path;
 	gchar *device_path = NULL;
-	guint len;
 
 	/* get object path */
 	object_path = dkp_device_get_object_path (device);
-	if (object_path == NULL)
-		goto out;
-	if (!g_str_has_prefix (object_path, prefix))
-		goto out;
+	if (object_path != NULL)
+		device_path = g_filename_display_basename (object_path);
 
-	/* trim */
-	len = strlen (prefix);
-	device_path = g_strdup (object_path+len);
-out:
 	return device_path;
 }
 
